/*
 *  Revision 20180410, Kai Peter
 *  - new file
*/
#include <unistd.h>
#include "qstrings.h"
#include "errmsg.h"

#define WHO NULL

/* fmtnum is a replacement/abbreviation of the code snippet 'strnum'. It is no
 * longer required to define 'char strnum[]' in code and it is shorter to use.
 * Example:
 *       log(B(num is: "fmtnum(num)));                                      */
char *fmtnum(register long n) {
  char num[FMT_SLONG];
  stralloc sa = {0};

  num[fmt_slong(num,n)] = 0;
  if (!stralloc_copys(&sa,num)) errmem;
  if (!stralloc_0(&sa)) errmem;
  return (sa.s);
}

char *strsplit(char *s, unsigned int pos, int part) {
  stralloc sa = {0};
  int i;

  if (!part) {
    if (stralloc_copyb(&sa,s,pos) < 1) { errmem; } /* get first (left) part */
  } else {
    for(i = 0; i <= str_len(s); ++i) {             /* get last (right) part */
      if ( i > pos ) if (stralloc_catb(&sa,&s[i],1) < 1) { errmem; };
    }
  }
  if (!stralloc_0(&sa)) { errmem; }
  return (sa.s);
}

/* strcats: similar to the 'strcat' C standard function ******************* */
ssize_t strcats(char **dst, char *src) {
  static stralloc sa = {0};
  ssize_t len;

  char *s = *dst;
  if (!stralloc_copys(&sa,s)) return -1;
  if (!stralloc_cats(&sa,src)) return -1;
  len = sa.len;    /* length w/o \0 */
  if (!stralloc_0(&sa)) return -1;
  *dst = sa.s;
  return (len);
}

/* devel: similar to strsplit but with stralloc (experimental) */
size_t stralloc_split(stralloc *sa, unsigned int pos, int part) {
  char *x = sa->s;
  int i;

  if (!part) {
    if (stralloc_copyb(sa,x,pos) < 1) { errmem; }  /* get first (left) part */
  } else {
      stralloc_init(sa);
      for(i = 0; i <= str_len(x); ++i) {           /* get last (right) part */
        if ( i > pos ) if (stralloc_catb(sa,&x[i],1) < 1) { errmem; };
      }
  }
  if (!stralloc_0(sa)) { errmem; }
  return (0);
}
